"
COPYRIGHT (c) 2020-2021 LabWare
"
"{ Package: 'stx:goodies/monticello' }"

"{ NameSpace: Smalltalk }"

Object subclass:#MCSnapshotOptions
	instanceVariableNames:'includeExtrasForSTX porter'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Base'
!

!MCSnapshotOptions class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020-2021 LabWare


"
! !

!MCSnapshotOptions methodsFor:'accessing'!

includeExtrasForSTX    
    
    "/ do not default to true here, as the version is snapshotted twice in order to
    "/ update the version-strings, and we don not need this stuff in the first round!!
    ^ includeExtrasForSTX ? false

    "Modified: / 07-09-2015 / 15:22:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

includeExtrasForSTX:aBoolean
    includeExtrasForSTX := aBoolean.
!

porter
    "Return an instance of source porter (a kind of AutomagicSourcePorter)
     to apply to method's source when writing a snapshot. 

     Defaults to use base AutomagicSourcePorter instance to:

      * Strip namespaces if any
      * Replace end-of-line comments with normal one.
      * Replace (St/X) 'Block' with (Pharo / Squeak / Cuis) 'BlockClosure'

     Nor Squeak nor Pharo does support either of the above, so this is
     safe and convenient choice."

    porter isNil ifTrue:[ 
        porter := AutomagicSourcePorter new.
        "/ JV@2021-05-04: not sure this is a good idea,
        "/ but all the MC-related code is used almost 
        "/ exclusively in Squeak / Pharo / Cuis world, so
        "/ which have BlockClosure, not Block like St/X.
        "/ 
        "/ Monticello / Tonel is also used with GS but there
        "/ BlockClosure is an abstract superclass, so the rewrite
        "/ is also valid.
        "/ 
        "/ For the sake of completenes:
        "/ In VA, there's no BlockClosure, they call it `Block`. But then, 
        "/ in VA Tonel or Monticello is not used that extensively
        "/ and if so, they would have to have a way to deal with that
        "/ anyway.
        porter klassMap at: #Block put: #BlockClosure.
    ].
    ^ porter

    "Modified: / 04-05-2021 / 13:03:09 / Jan Vrany <jan.vrany@labware.com>"
    "Modified (comment): / 04-05-2021 / 22:05:18 / Jan Vrany <jan.vrany@labware.com>"
!

porter:porterInstance   
    "
    Set the porter instance to use when writing snapshot. Set to
    `nil` to disable any source code rewriting.

    For details, see #porter comment.
    "
    porter := porterInstance.

    "Modified (comment): / 02-07-2020 / 21:24:12 / Jan Vrany <jan.vrany@labware.com>"
! !

!MCSnapshotOptions class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

