"{ Package: 'stx:goodies/monticello/tonel' }"

"{ NameSpace: Smalltalk }"

TonelScanner subclass:#TonelSourceScanner
	instanceVariableNames:'char prevChar result count isFinished'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Tonel-Core'
!

!TonelSourceScanner methodsFor:'private'!

prepareToScan
	result := String new writeStream.
	isFinished := false.
	char := prevChar := nil.
	count := 0
	
!

readNext
	prevChar := char.
	result nextPut: (char := stream next)
!

readUntil: aBlock
	[ stream atEnd ]
	whileFalse: [
		self readNext.
		aBlock value ifTrue: [ ^ self ] ]
!

readUntilTermination: matchChar
	| terminationCount |
	"This is used to read sequences that start and finish with same character termination. 
	 tipically, this is for strings and comments, but who knows what comes in the future."
	terminationCount := 1.
	self readUntil: [ 
		char = matchChar ifTrue: [ terminationCount := terminationCount + 1 ].
		terminationCount even and: [ stream peek ~= matchChar ] ]
! !

!TonelSourceScanner methodsFor:'scanning'!

scan
	self prepareToScan.	
	stream peek = $[ ifFalse: [ TonelParseError signal: 'Can''t parse method body' ].
	[ stream atEnd or: [ isFinished ] ]
	whileFalse: [ self scanNextChunk ].
	isFinished ifTrue: [ 
		"clean up to return"
		^ self 
			removeFrom: result contents withSqueakLineEndings
			enclosingStart: $[ 
			end: $]
			clean: #right ].

	TonelParseError signal: 'Can''t parse method body'
!

scanForBlock
	| match |
	match := false.
	char = $[ ifTrue: [ 
		count := count + 1.
		match := true ].
	char = $] ifTrue: [ 
		count := count - 1.
		match := true ].
	isFinished := count = 0.
	^match
!

scanForComment
	self isStartingComment ifFalse: [ ^ false ]. 
	self readUntilTermination: $".
	^true
!

scanForLiteralArray
	| literalCount |
	self isStartingLiteralArray ifFalse: [ ^ false ]. 
	literalCount := 1.
	self readUntil: [
		self scanForLiteralCharacter
			or: [ self scanForString
			or: [ self scanForComment
			or: [ char = $( ifTrue: [ literalCount := literalCount + 1 ].
					char = $) ifTrue: [ literalCount := literalCount - 1 ]]]].
		literalCount = 0 ].
	^true
!

scanForLiteralCharacter
	self isStartingLiteralCharacter ifFalse: [ ^ false ]. 
	self readNext.
	^true
!

scanForString
	self isStartingString ifFalse: [ ^ false ].
	self readUntilTermination: $'.
	^true
!

scanNextChunk
	self readNext.	
	self scanForBlock or: [
	self scanForComment or: [
	self scanForString or: [
	self scanForLiteralCharacter or: [
	self scanForLiteralArray ]]]]
! !

!TonelSourceScanner methodsFor:'testing'!

isStartingComment
	"Comment start with a double quote in Smalltalk syntax"
	^ char = $"
!

isStartingLiteralArray
	"Literal array starts with a sharp sign followed by parenthesis in Smalltalk syntax"
	^ char = $( and: [ prevChar = $# ]
!

isStartingLiteralCharacter
	"A literal Character start with a dollar sign in Smalltalk syntax"  
	^ char = $$
!

isStartingString
	"String start with single quote in Smalltalk syntax"
	^ char = $'
! !

